;
; BLUE STAR
; A game for the unexpanded Vic-20
; (c) 2008 Steve McCrea
; Written Christmas 2007/January 2008
;
; use ACME to assemble - no arguments
; load in Vice's monitor with bl "bluestar.obj" 0 1001
; or make a D64 from the prg and autostart it
; then RUN

; !to "bluestar.obj", plain
!to "bluestar_unc.prg", cbm
!sl "bluestar.lbl"   ; dump labels

!source "graphics.asm"

*=$1001

; global defines
    chardefs        = $1c00
    screen          = $1e00
    screencols      = $9600
    lowerscreen     = screen + $100
    lowerscreencols = screencols + $100
    
    OFFSET_XCOORD       = 0
    OFFSET_YCOORD       = 1
    OFFSET_SPRITEOFFSET = 2
    OFFSET_HEIGHT       = 3
    OFFSET_FLAGS        = 4
    OFFSET_XADD         = 5
    OFFSET_YADD         = 6
    OFFSET_HEALTH       = 7
    OFFSET_DAMAGETIME   = 8
    STRUCTSIZE          = 9
    
    FLAGS_XCOLLISION    = 1
    FLAGS_YCOLLISION_UP = 2
    FLAGS_YCOLLISION_DN = 4
    FLAGS_ACTIVE        = 8
    FLAGS_DAMAGED       = 16
    FLAGS_COLLIDABLE    = 32
    FLAGS_EXPLODE       = 64
    
    ABILITY_COUNT       = 4
    
    CHARTYPE_FLOATER    = 0
    CHARTYPE_CRAWLER    = 1
      ;
      ;
    CHARTYPE_REACTOR    = 4
    CHARTYPE_BULLET     = 5
    CHARTYPE_EXPLOSION  = 6
    CHARTYPE_PLAYER     = 7
    
    CHARINDEX_BULLET    = 0
    CHARINDEX_FIRSTAI   = 1
    CHARINDEX_PLAYER    = 5
    CHAR_COUNT          = 6
    
    RASTERLINE_PAL      = $66
    RASTERLINE_NTSC     = $54
    
    AUXCOLOR_ORANGE     = $80 ; for poking to $900E
    
;
; ZP allocations
; can use them all since I don't want interrupts
; extra 256 bytes!
;
; character structure
    xcoord          = $04
    ycoord          = $05
    spriteoffset    = $06
    height          = $07
    flags           = $08
    xadd            = $09
    yadd            = $0a
    health          = $0b
    damagetime      = $0c
    structsize      = $0d
    
    currentchar     = $0f
    CharacterType   = $10 ; length 7
    CollidedBlockType = $17
    CollidedBlockPos = $18
    PlayerRoom      = $19
    NextRoom        = $1a
    randomtemp      = $1b
    randomlo        = $1c
    randomhi        = $1d
    MusicIndex      = $1e
    MusicToggleNote = $1f
    MusicPtr        = $20   ; 2 bytes
    MusicDelay      = $22
    SfxDuration     = $23
  SectionToZeroOut = $24
    NumRoomsVisited = $24
    PlayerCanDoubleJump = $25
    PlayerAbilities = $26 ; length 4
    PlayerHasBoost  = $26
    PlayerHasMissile = $27
    PlayerHasCodeA  = $28
    PlayerHasCodeB  = $29
    PlayerNumFramesOffFloor = $2a
    JumpIsPressed   = $2b
    JumpPressedThisFrame = $2c
    FireIsPressed   = $2d
    FirePressedThisFrame = $2e
    ReactorPiecesDestroyed = $2f
    gametime        = $30 ; length 3
    updatecount     = $33
  EndSectionToZeroOut = $33
    lownibble       = $34
    highnibble      = $35
    rasterline      = $36
    stickleft       = $37
    stickright      = $38
    stickup         = $39
    stickfire       = $3a
    stickcontribute = $3b

    Characters      = $3c
    BulletCharacter = Characters
    AICharacters    = BulletCharacter + STRUCTSIZE
    PlayerCharacter = AICharacters + 4*STRUCTSIZE
    
    ts              = $72 ; start of temporaries
    ColorsUsed      = $80 ; length 25


;
; To move and render characters...
; Each character has a struct which I will copy to the zero page
; Then copy back when done
;
    
; global variables

; basic program to exec: 2008 SYS 7680
!word $100b     ; pointer to next line
!word 2008      ; line number
!byte $9e       ; SYS
!text "7680", 0 ; 7680
!word 0         ; pointer to next line

Main
TitleLoop
    jsr DrawTitleScreen
-
    jsr WaitForRasterLineLessThan
    jsr WaitForRasterLine     ; bottom of graphics part of screen
    jsr CheckSpace
    bne -
-   jsr CheckSpace
    beq -
GameStart
    jsr SetupPlayer
    jsr SetupRoom
    ldx #<MusicTrack
    ldy #>MusicTrack
    jsr StartMusic

-   
    jsr UpdateMusic
    jsr UpdateSfx
    jsr Update
    jsr WaitForRasterLineLessThan ; in case the update was quick
    jsr WaitForRasterLine     ; bottom of graphics part of screen
    jsr UpdateMusic
    jsr UpdateSfx
    jsr Render
    jsr WaitForRasterLineLessThan ; in case the render was quick
    jsr WaitForRasterLine
    jsr EraseAndCopyUpSprites
    lda PlayerCharacter+OFFSET_FLAGS
    and #FLAGS_ACTIVE
    beq TitleLoop
    lda ReactorPiecesDestroyed
    cmp #4
    bne -
    jsr CheckSpace
    bne -
-   jsr CheckSpace
    beq -
    bne TitleLoop

;
; Update
;

Update
    ;lda #$0c
    ;sta $900f   ; change border color
    lda NextRoom
    bmi +
    sta PlayerRoom
    lda #-1
    sta NextRoom
    jsr SetupRoom
+
    lda ReactorPiecesDestroyed
    cmp #4
    beq +

    clc
    lda updatecount
    adc #1
    sta updatecount

    sed
    ldx #2
-   lda gametime, x
    adc #0
    sta gametime, x
    dex
    bpl -
    cld

    lda #STRUCTSIZE
    sta structsize
    jsr UpdateCharacters
+
    ;lda #$0a
    ;sta $900f   ; change border color back
    rts

;
; Render
;

Render
    ;lda #$0b
    ;sta $900f   ; change border color
    jsr StartRenderSprites
    lda #5
    sta structsize
    lda #0
    sta currentchar
-   jsr CopyCharacterToZeroPage
    lda flags
    and #FLAGS_ACTIVE
    beq +
    jsr RenderSprite
+   inc currentchar
    jsr CheckRasterLineLessThan
    lda currentchar
    cmp #CHAR_COUNT
    bne -
    rts

!source "utils.asm"
!source "input.asm"
!source "simplemusic.asm"
!source "chars.asm"
!source "creatures.asm"
!source "player.asm"
!source "rooms.asm"
!source "collision.asm"
!source "sprite.asm"
!source "title.asm"

EndOfCode

; these need to be the last thing sourced as they play with the PC
!source "roomdata.asm"
!source "strings.asm"

* = $1e00

; put throwaway code here
    ; don't need to disable interrupts - pucrunch will leave them disabled
    ; sei             ; don't need or want interrupts!

    ldx #$ff        ; reset the stack so we can use most of $100-$1ff
    txs             ; the stack will never get below $1f0

    lda $ede4    ; load ROM horizontal screen origin
    cmp #5       ; if 5, it's NTSC
    beq +
    lda #RASTERLINE_PAL
    ldx #0
    bpl ++
+   lda #RASTERLINE_NTSC
    ldx #4
++  sta rasterline
    ldy #3
-   lda VicContents, x
    sta $9000, y
    inx
    dey
    bpl -

    lda #(AUXCOLOR_ORANGE)  ; set the auxiliary color
    sta $900e
    
RelocateRooms
    ; copy the entire block
    ldx #0
-   lda StartOfRoomsUnrelocated, x
    sta StartOfRooms, x
    inx
    bne -

RelocateSprites
    ldx #(EndOfSprites-Sprites)
-   lda StartOfSpritesUnrelocated-1, x
    sta Sprites-1, x
    dex
    bne -

    jmp $100d

; $9000 = center screen X (leave top bit clear)
; $9001 = center screen Y
; $9002 = screen width 16 (leave top bit set)
; $9003 = screen height 24 (leave bottom bit clear)

    ; make the screen 16x24, and adjust the position
VicContents ; these are backwards, to make the code shorter
VicContentsPAL  !byte 24<<1, $80+16, 38, 18
VicContentsNTSC !byte 24<<1, $80+16, 20, 10
