;
; IsCollidingWithChar
;
; IN:
; .X is the character to collide with the zero page character
; OUT:
; C=collision
;

IsCollidingWithChar
    ldy CharacterLookup,x
    lda Characters + OFFSET_FLAGS,y
    and #(FLAGS_ACTIVE + FLAGS_COLLIDABLE)
    cmp #(FLAGS_ACTIVE + FLAGS_COLLIDABLE)
    bne ++
    lda Characters + OFFSET_XCOORD,y
    sec
    sbc xcoord
    bpl +
    eor #$ff
    clc
    adc #1
+   cmp #4
    bpl ++
    lda ycoord
    clc
    adc height
    sec
    sbc Characters + OFFSET_YCOORD,y
    bmi ++
    beq ++
    lda Characters + OFFSET_YCOORD,y
    clc
    adc Characters + OFFSET_HEIGHT,y
    sec
    sbc ycoord
    bmi ++
    beq ++
    sec
    rts
++  clc
    rts

;
; CollideX
;
; Move and collide a character with what is on the screen
; movement in the x direction only
; Collidable blocks are chars FIRST_COLLIDABLE and up
;

; temps
    upperscreenrow = ts
    lowerscreenrow = ts+1
    oldcol = ts+2
    newcol = ts+3
    partialx = ts+4 ; 0 when moving left, 3 when moving right

CollideX
    lda xadd
    beq .x_clear_and_ret
    lda ycoord    ; upperscreenrow = y&f8<<1
    and #$f8
    asl
    sta upperscreenrow
    ldy height    ; lowerscreenrow = (y+h-1)&f8<<1
    dey
    tya
    clc
    adc ycoord
    and #$f8
    asl
    sta lowerscreenrow
    lda #0
    ldx xadd
    bmi +
    lda #3
+   sta partialx
    clc
    adc xcoord
    tay
    lsr
    lsr
    sta oldcol
    tya
    clc
    adc xadd
    lsr
    lsr
    sta newcol
    cmp oldcol
    bne .collide_x_maybe
.collide_x_no_collision
    lda xcoord
    clc
    adc xadd
    sta xcoord
.x_clear_and_ret
    lda flags
    and #(!FLAGS_XCOLLISION)
    sta flags
    rts
.collide_x_maybe
    clc
    adc upperscreenrow
    tay
    lda $1e00,y
    cmp #FIRST_COLLIDABLE
    bpl .x_collision
    lda newcol
    clc
    adc lowerscreenrow
    tay
    lda $1e00,y
    cmp #FIRST_COLLIDABLE
    bmi .collide_x_no_collision
.x_collision
    sta CollidedBlockType
    sty CollidedBlockPos
    lda flags
    ora #FLAGS_XCOLLISION
    sta flags
    lda xadd
    bpl +
    lda xcoord
    and #$fc
    sta xcoord
    rts
+   lda newcol
    sec
    sbc #1
    asl
    asl
    sta xcoord
    rts

;
; CollideY
;
; Move and collide a character with what is on the screen
; movement in the y direction only
; Collidable blocks are chars FIRST_COLLIDABLE and up
;

; temps
    leftscreencolumn = ts
    rightscreencolumn = ts+1
    oldrow = ts+2
    newrow = ts+3
    partialy = ts+4 ; 0 when moving up, height - 1 when moving down

CollideY
    lda yadd
    beq .y_clear_and_ret
    lda xcoord
    lsr
    lsr
    sta leftscreencolumn
    lda xcoord
    clc
    adc #$03
    lsr
    lsr
    sta rightscreencolumn
    lda #0
    ldy yadd
    bmi +
    ldy height
    dey
    tya
+   sta partialy
    clc
    adc ycoord
    tay
    and #$f8
    asl
    sta oldrow
    tya
    clc
    adc yadd
    and #$f8
    asl
    sta newrow
    cmp oldrow
    bne .possible_y_collision
.no_y_collision
    lda ycoord
    clc
    adc yadd
    sta ycoord
.y_clear_and_ret
    lda flags
    and #(!(FLAGS_YCOLLISION_UP + FLAGS_YCOLLISION_DN))
    sta flags
    rts
.possible_y_collision
    clc
    adc leftscreencolumn
    tay
    lda $1e00,y
    cmp #FIRST_COLLIDABLE
    bpl .collide_y_collision
    lda newrow
    clc
    adc rightscreencolumn
    tay
    lda $1e00,y
    cmp #FIRST_COLLIDABLE
    bmi .no_y_collision
.collide_y_collision
    lda flags
    ldy yadd
    bpl +
    ora #FLAGS_YCOLLISION_UP
    sta flags
    lda oldrow  ; ycoord = oldrow>>1
    lsr
    sta ycoord
    rts
+   ora #FLAGS_YCOLLISION_DN
    sta flags
    lda newrow  ; ycoord = (newrow>>1) - height
    lsr
    bne +
    lda #128    ; going off screen (wraps at 128)
+   sec
    sbc height
    sta ycoord
    rts

