;
; SetupPlayer
;

SetupPlayer
    lda #CHARTYPE_PLAYER
    sta CharacterType+CHARINDEX_PLAYER
    ldx #(STRUCTSIZE-1)
-   lda PlayerInitialValues,x
    sta PlayerCharacter,x
    dex
    bpl -
    lda #32
    sta PlayerRoom
    lda #-1
    sta NextRoom
    lda #0
    ldx #(EndSectionToZeroOut - SectionToZeroOut)
-   sta SectionToZeroOut,x
    dex
    bpl -
; TEMPORARY - remove strings to make space
;    lda #1
;    sta PlayerHasMissile
;    lda #64
;    sta NumRoomsVisited
    rts

PlayerInitialValues
!byte $9    ; x
!byte 8*$7  ; y
!byte PlayerSprites - Sprites
!byte 8     ; height
!byte FLAGS_ACTIVE + FLAGS_COLLIDABLE
!byte 0     ; xadd
!byte 0     ; yadd
!byte 99    ; health
!byte 0     ; damagetime

;
; UpdatePlayer
;


UpdatePlayer
    jsr GetPlayerInput
    jsr UpdatePlayerPhysics
    jsr CollideX
    ldx CollidedBlockType
    cpx #TILE_FIRSTPICKUP
    bmi +
    lda #99
    sta health
    ldy CollidedBlockPos
    lda #0
    sta screen, y
    sta CollidedBlockType
    cpx #TILE_HEALTH
    beq +
    txa
    sec
    sbc #TILE_FIRSTPICKUP
    tay
    lda #1
    sta PlayerAbilities, y
    ldx AbilityStrings, y
    jsr DrawString
    ldx #(FoundString-StringBase)
    jsr DrawString
+   jsr CollideY
    jsr UpdatePlayerSprite
    jsr UpdatePlayerRoom
++  jsr UpdateDamage
    jsr DrawHUD
    rts


;
; GetPlayerInput
;

GetPlayerInput
    jsr ScanJoystick
        ; left
    lda #0
    sta xadd
    ldx #$f7        ; XVN
    ldy #$ff
    jsr ScanKeyRow
    ora stickleft
    beq +
    lda #$ff
    sta xadd
        ; right
+   ldx #$ef        ; ZCB
    ldy #$fe        ; not space
    jsr ScanKeyRow
    ora stickright
    clc
    adc xadd
    sta xadd
        ; jump
    ldx #$fb        ; ADG
    ldy #$df        ; SFH
    lda stickfire
    sta stickcontribute
    lda JumpIsPressed
    jsr ScanEntireKeyRow
    stx JumpIsPressed
    sty JumpPressedThisFrame
        ; fire
    ldx #$bf        ; QET
    ldy #$fd        ; WRY
    lda stickup
    sta stickcontribute
    lda FireIsPressed
    jsr ScanEntireKeyRow
    stx FireIsPressed
    sty FirePressedThisFrame

        ; use jump input
    lda JumpPressedThisFrame
    beq ++
    lda flags
    and #FLAGS_YCOLLISION_DN
    beq +
-   ldx #130
    ldy #8
    jsr PlaySfx
    lda #-7
    sta yadd
    bmi ++    ; always
+   lda PlayerNumFramesOffFloor
    cmp #3
    bmi -
    lda PlayerHasBoost
    and PlayerCanDoubleJump
    beq ++
    ldx #220
    ldy #8
    jsr PlaySfx
    lda #-7
    sta yadd
    lda #0
    sta PlayerCanDoubleJump
++
        ; use fire input
    lda FirePressedThisFrame
    and PlayerHasMissile
    beq .nofire
    lda BulletCharacter+OFFSET_FLAGS
    and #FLAGS_ACTIVE
    bne .nofire
    ldx #240
    ldy #4
    jsr PlaySfx
    lda #0
    sta BulletCharacter+OFFSET_YADD
    lda PlayerLastMovementDir
    asl
    sta BulletCharacter+OFFSET_XADD
    lda #3
    sta BulletCharacter+OFFSET_HEIGHT
    lda #(MissileSprites - Sprites)
    ldx PlayerLastMovementDir
    bpl +
    clc
    adc #4
+   sta BulletCharacter+OFFSET_SPRITEOFFSET
    lda xcoord
    sta BulletCharacter+OFFSET_XCOORD
    lda ycoord
    clc
    adc #2
    sta BulletCharacter+OFFSET_YCOORD
    lda #(FLAGS_ACTIVE + FLAGS_COLLIDABLE)
    sta BulletCharacter+OFFSET_FLAGS
    lda #CHARTYPE_BULLET
    sta CharacterType + CHARINDEX_BULLET
.nofire

    rts

;
; UpdatePlayerBullet
;

UpdatePlayerBullet
    lda flags
    and #FLAGS_EXPLODE
    bne .explodebullet
    jsr CollideX
    lda flags
    and #FLAGS_XCOLLISION
    beq .endbullet
        ; check if we hit a door
    lda CollidedBlockType
    cmp #TILE_DOOR
    bne .explodebullet
        ; yup, open the door
    ldx CollidedBlockPos
    lda screencols, x
    and #$0f
    cmp #CYAN
    bne .opendoorandexplodebullet
        ; special case swipe card door
    ldx #(CodesString-StringBase)
    jsr DrawString
    lda PlayerHasCodeA
    and PlayerHasCodeB
    beq .bouncebullet
    ldx #(UsedString-StringBase)
    jsr DrawString
    jmp .opendoorandexplodebullet
.bouncebullet
    lda #-1
    sta xadd
    lda #(MissileSprites - Sprites + 4)
    sta spriteoffset
    ldx #(NeededString-StringBase)
    jsr DrawString
    rts
.opendoorandexplodebullet
    lda #0
    ldx CollidedBlockPos
    sta screen, x
.explodebullet
    lda ycoord
    sec
    sbc #3
    sta ycoord
    jsr ExplodeThisCharacter
.endbullet
    rts

;
; UpdatePlayerPhysics
;

PlayerLastMovementDir !byte 1

UpdatePlayerPhysics
    lda xadd
    beq +
    sta PlayerLastMovementDir

+   lda flags
    and #(FLAGS_YCOLLISION_UP + FLAGS_YCOLLISION_DN)
    beq ++
    lda flags
    and #FLAGS_YCOLLISION_DN
    beq +
        ; hit below (standing on something)
    lda #1
    sta PlayerCanDoubleJump
    sta PlayerNumFramesOffFloor
    lda yadd
    bmi ++
-   lda #0
    sta yadd
    beq ++ ; always
        ; hit above
+   lda yadd
    bmi -
++  inc PlayerNumFramesOffFloor
    ldy yadd
    cpy #7
    bne +
    rts
+   iny
    sty yadd
    rts

;
; UpdatePlayerSprite
;

UpdatePlayerSprite
    lda xcoord
    and #4
    lsr
    tax
    lda xadd
    beq ++
    bpl +
    inx
+   lda TimesNine, x
    sta spriteoffset
++  rts

;
; UpdatePlayerRoom
;

UpdatePlayerRoom
    lda xcoord
    bne +
    lda #56
    sta xcoord
    ldx PlayerRoom
    dex
    bpl ++

+   cmp #60
    bne +
    lda #4
    sta xcoord
    ldx PlayerRoom
    inx
++  stx NextRoom
    rts

+   lda ycoord
    bne +
    lda #112
    sta ycoord
    lda PlayerRoom
    sec
    sbc #8
    bpl ++

+   cmp #116
    bmi +
    lda #8
    sta ycoord
    lda PlayerRoom
    clc
    adc #8
++  sta NextRoom
+   rts


;
; DrawHUD
;

DrawHUD

;
; DrawHealthBar
;
; part of DrawHUD, so no RTS
; temps
    healthbarlength = ts

DrawHealthBar
    lda CharacterType+CHARINDEX_PLAYER
    cmp #CHARTYPE_PLAYER
    bne ++
+   ldy #12
    ldx #0
    lda PlayerCharacter+OFFSET_HEALTH
    lsr
    lsr
    lsr
    bmi +
    beq +
    sta healthbarlength
-   lda #(128 + 116)
    sta lowerscreen + 18, x
    lda #(MAGENTA-8)
    sta lowerscreencols + 18, x
    inx
    dey
    cpx healthbarlength
    bne -
+   lda #32
-   sta lowerscreen + 18, x
    inx
    dey
    bpl -
++

;
; DrawPlayerRoomNumber
;
; part of DrawHUD, so no RTS

DrawPlayerRoomNumber
    lda PlayerRoom
    lsr
    lsr
    lsr
    clc
    adc #1 ; 'A'
    sta lowerscreen+$3b
    lda PlayerRoom
    and #7
    adc #49 ; '1'
    sta lowerscreen+$3c
    lda #(WHITE-8)
    sta lowerscreencols+$3b
    sta lowerscreencols+$3c

;
; PrintTime
;

PrintTime
    ldx #0
    ldy #$33
-   lda #(WHITE-8)
    sta lowerscreencols,y
    sta lowerscreencols+1,y
    lda gametime,x
    jsr SplitIntoNibbles
    clc
    adc #48 ; '0'
    sta lowerscreen, y
    iny
    lda lownibble
    adc #48 ; '0'
    sta lowerscreen, y
    iny
    inx
    cpx #3
    bne -

    rts

