;
; SetupRoom
;
; IN:
;
; temps
    roomaddress = ts ; & ts+1
    length = ts+2
    direction = ts+3
    tiletype = ts+4
    tilepos = ts+5
    tilecolor = ts+6
    doorpos = highnibble
    tileand = ts+7
    doortype = ts+8
    creatureplus = ts+9
    maxcreatureindex = ts+10
    

    SH_X = 4
    XNONE = 0
    XR = 1<<SH_X
    XU = 2<<SH_X
    XUR = 3<<SH_X
    XL = 4<<SH_X
    XLR = 5<<SH_X
    XLU = 6<<SH_X
    XLUR = 7<<SH_X
    XD = 8<<SH_X
    XDR = 9<<SH_X
    XDU = 10<<SH_X
    XDUR = 11<<SH_X
    XDL = 12<<SH_X
    XDLR = 13<<SH_X
    XDLU = 14<<SH_X
    XDLUR = 15<<SH_X
    
    TRL = 0
    TRR = 1
    SSH = 2
    DIA = 3
    LSH = 4
    TRE = 5
    SPL = 6
    ZSH = 7
    HSH = 8
    CAV = 9
    BOX = 10
    END = 11

RoomAddresses
!word Room0 ; treasure left
!word Room1 ; treasure right
!word Room2 ; S shape
!word Room3 ; diagonal
!word Room4 ; L shape
!word Room5 ; tree
!word Room6 ; scattered platforms
!word Room7 ; Z shape
!word Room8 ; H shape
!word Room9 ; cave
!word BoxRoom
!word EndRoom

BOXROOM = 10

RoomMap
!byte $ff,    $ff,     XDR+ZSH, XLR+LSH,XNONE+END,XR+TRL, XDL+TRR,  XD+TRL
!byte XR+TRL, XDL+DIA, XUR+SSH, XDLR,    XL+TRR,  XDR+TRL,XLU+DIA,  XDU+HSH
!byte XDR+SPL,XDLUR+HSH,XL+TRR, XUR+LSH, XDLR+TRL,XLU+DIA,XR+CAV,   XDLU+SPL
!byte XU+TRE, XUR+ZSH, XDL,     XDR+DIA, XLUR+HSH,XDLR,   XLR+CAV,  XLU+SSH
!byte XR+TRL, XDLR+LSH,XLU+HSH, XUR+TRE, XDL+TRL, XUR+SPL,XLR+TRR,  XDL+ZSH
!byte XDR+TRL,XLUR+TRE,XDLR+LSH,XLR+CAV, XLU+SSH, XDR+SPL,XLR+CAV,  XDLU+HSH
!byte XU+SSH, XDR+DIA, XDLU+SPL,XDR+TRL, XL+TRR, XDUR+ZSH,XDL,      XU+TRE
!byte XR+TRL, XLU+ZSH, XUR+TRE, XLU+SPL, XR+TRL, XLUR+TRE,XLUR+DIA, XL+TRR

PickupRooms
!byte 56, 8, 60, 7
HealthRooms
!byte 5, 12, 18, 22, 24, 36, 48, 52, 55, 63
HealthPositions
!byte $74, $2d, $2d, $e2, $ed, $74, $e2, $2d, $ed, $2d

DirectionLookup !byte 1,16,-15,17

CharacterHealth
!byte 5, 10, 15, 20, 25    ; Bat, BloodMonkey, JellySkull, Lionix, Reactor
CharacterSpriteOffset
!byte BatSprites-Sprites, BloodMonkeySprites-Sprites, JellySkullSprites-Sprites, LionixSprites-Sprites, ReactorSprite-Sprites

TileColors
!8  GREEN, BLUE, GREEN, YELLOW
!8  BLUE, BLUE, BLUE, GREEN

SetupRoom
    jsr ClearScreen
    jsr ResetSprites
    lda #RED       ; border = red, background = black, no inverse
    sta $900f
    ldx PlayerRoom
    lda RoomMap,x
    and #$0f
    jsr SetupRoomA
    lda #BOXROOM
    jsr SetupRoomA
    jsr SetupDoors
    jsr SetupPickups
    inc NumRoomsVisited
    rts

SetupRoomA
    asl
    tax
    lda RoomAddresses,x
    sta roomaddress
    lda RoomAddresses+1,x
    sta roomaddress+1

    jsr DrawTileStrips
    
        ; special case don't read creatures
    lda (roomaddress), y
    bne +
    rts
+

;    lda #1
 ;   sta NumCharsUsed
        ; deactivate creatures
    ldx #(4*STRUCTSIZE)
    sec
-   lda #0
    sta BulletCharacter+OFFSET_FLAGS,x
    txa
    sbc #STRUCTSIZE
    tax
    bpl -
    
        ; creatures at the right of the map are +2
+   lda PlayerRoom
    and #4
    lsr
    sta creatureplus
        ; max creatures depends on how many rooms visited
        ; starts at 1 and increases by 1 every 8 rooms
    lda NumRoomsVisited
    lsr
    lsr
    lsr
    clc
    adc #(CHARINDEX_FIRSTAI+1)
    sta maxcreatureindex
        ; load creatures
    lda #STRUCTSIZE
    sta structsize
    lda #CHARINDEX_FIRSTAI
    sta currentchar
-   lda (roomaddress), y
    iny
    cmp #$ff
    beq +
    pha
    and #$0f
    asl
    asl
    sta xcoord
    pla
    and #$f0
    lsr
    sta ycoord
    lda #(FLAGS_ACTIVE + FLAGS_COLLIDABLE)
    sta flags
    ldx currentchar
    lda (roomaddress), y
    iny
    clc
    adc creatureplus
    sta CharacterType, x
    tax
    lda CharacterHealth, x
    sta health
    lda #8
    sta height
    lda CharacterSpriteOffset, x
    sta spriteoffset
    lda #0
    sta xadd
    sta yadd
    sta damagetime
    tya
    pha
    jsr CopyCharacterFromZeroPage
    pla
    tay
    inc currentchar
    lda currentchar
    cmp maxcreatureindex
    beq +
    bne -

+   rts

    
;
; SetupDoors
;

SetupDoors
    ldy #0
    lda PlayerRoom
    tax
    cmp #27 ; special case missile entrance
    beq +
    and #%100100
    cmp #%000100 ; check if in top right corner of map
    bne ++
+   ldy #TILE_DOOR
++  sty doortype
    lda RoomMap,x
    jsr SplitIntoNibbles
    and #1
    beq +
    ldx doortype
    stx screen+$ef
    ldx #WHITE
    stx screencols+$ef
+   lda doorpos
    and #2
    beq +
    ldx #0
    stx screen+5
    stx screen+6
    stx screen+9
    stx screen+10
+   lda doorpos
    and #4
    beq +
    ldx doortype
    stx screen+$e0
    ldx #WHITE
    stx screencols+$e0
+   lda doorpos
    and #8
    beq +
    ldx #0
    stx screen+$f7
    stx screen+$f8
+   lda PlayerRoom    ; special case swipe card door
    cmp #2
    bne +
    lda #TILE_DOOR
    sta screen+$ef
    lda #CYAN
    sta screencols+$ef
+   rts

;
; DrawTileStrips
;

DrawTileStrips
        ; rooms on the RHS of the map are different tileset
    lda PlayerRoom
    and #4
    sta tileand
    ldy #0
--  lda (roomaddress),y
    iny
    cmp #$ff
    beq +
    sta tilepos
    lda (roomaddress),y
    iny
    pha
    and #$03
    ora tileand
    sta tiletype
    pla
    pha
    lsr
    lsr
    and #$03
    sta direction
    pla
    lsr
    lsr
    lsr
    lsr
    sta length
-   ldx tiletype
    lda TileColors,x
    sta tilecolor
    txa
    clc
    adc #FIRST_COLLIDABLE
    ldx tilepos
    sta screen,x
    lda tilecolor
    sta screencols,x
    ldx direction
    lda DirectionLookup,x
    clc
    adc tilepos
    sta tilepos
    dec length
    bpl -
    bmi --
+
    rts

SetupPickups
    ldx #3
-   lda PickupRooms, x
    cmp PlayerRoom
    bne +   ; wrong room
    lda PlayerAbilities, x
    bne +   ; already have the ability
    txa
    clc
    adc #TILE_FIRSTPICKUP
    sta screen+$74
    lda #CYAN
    sta screencols+$74
+   dex
    bpl -

    ldx #9
-   lda HealthRooms, x
    cmp PlayerRoom
    bne +
    ldy HealthPositions, x
    lda #TILE_HEALTH
    sta screen, y
    lda #CYAN
    sta screencols, y
+   dex
    bpl -
    rts

